@@ -1,5 +1,5 @@
 /////////////////////////////////////////////////////////////////////////
-// $Id: cdrom.cpp,v 1.30 2004/02/02 17:01:18 milan Exp $
+// $Id: cdrom.cpp,v 1.31 2004/07/26 18:23:41 milan Exp $
 /////////////////////////////////////////////////////////////////////////
 //
 //  Copyright (C) 2002  MandrakeSoft S.A.
@@ -82,7 +82,7 @@ extern "C" {
 #include "cdrom_beos.h"
 #define BX_CD_FRAMESIZE 2048
 
-#elif (defined (OS_netbsd) || defined(OS_openbsd) || defined(OS__freebsd))
+#elif (defined (OS_netbsd) || defined(OS_openbsd) || defined(OS_freebsd))
 // OpenBSD pre version 2.7 may require extern "C" { } structure around
 // all the includes, because the i386 sys/disklabel.h contains code which 
 // c++ considers invalid.
