@@ -19,8 +19,7 @@
 
 #define sdlscrn		hostScreen.getPhysicalSurface()
 
-static SDL_Surface *stdfontgfx=NULL;
-static SDL_Surface *fontgfx=NULL;   /* The actual font graphics */
+static SDL_Surface *fontgfx=NULL;
 static int fontwidth, fontheight;   /* Height and width of the actual font */
 
 // Stores current dialog coordinates
@@ -39,42 +38,64 @@ enum
   SG_BCKGND_RECT_END
 };
 
+SDL_Color blackc[]     = {{0, 0, 0, 0}};
+SDL_Color darkgreyc[]  = {{128, 128, 128, 0}};
+SDL_Color greyc[]      = {{192, 192, 192, 0}};
+SDL_Color whitec[]     = {{255, 255, 255, 0}};
+
+typedef struct
+{
+  int object;
+  int position;
+  int blink_counter;
+  bool blink_state;
+} cursor_state;
+
+enum
+{
+  SG_FIRST_EDITFIELD,
+  SG_PREVIOUS_EDITFIELD,
+  SG_NEXT_EDITFIELD,
+  SG_LAST_EDITFIELD
+};
+
 /*-----------------------------------------------------------------------*/
 /*
-  Load an XBM into a SDL_Surface.
+  Load an 1 plane XBM into a 8 planes SDL_Surface.
 */
-static SDL_Surface *SDLGui_LoadXBM(int w, int h, uint8 *bits)
+static SDL_Surface *SDLGui_LoadXBM(int w, int h, Uint8 *srcbits)
 {
-	SDL_Surface *bitmap;
-	uint8 *line;    /* Allocate the bitmap */
-	bitmap = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 1, 0, 0, 0, 0);
-	if ( bitmap == NULL ) {
-		panicbug("Couldn't allocate bitmap: %s", SDL_GetError());
-		return(NULL);
-	}
-	/* Copy the pixels */
-	line = (uint8 *)bitmap->pixels;
-	w = (w+7)/8;
-	while ( h-- ) {
-		memcpy(line, bits, w);
-		/* X11 Bitmap images have the bits reversed */
-		{
-			int i, j;
-			uint8 *buf;
-			uint8 byte;
-			for ( buf=line, i=0; i<w; ++i, ++buf ) {
-				byte = *buf;
-				*buf = 0;
-				for ( j=7; j>=0; --j ) {
-					*buf |= (byte&0x01)<<j;
-					byte >>= 1;
-				}
-			}
-		}
-		line += bitmap->pitch;
-		bits += w;
-	}
-	return(bitmap);
+  SDL_Surface *bitmap;
+  Uint8 *dstbits;
+  int x, y, srcpitch;
+
+  /* Allocate the bitmap */
+  bitmap = SDL_CreateRGBSurface(SDL_SWSURFACE, w, h, 8, 0, 0, 0, 0);
+  if ( bitmap == NULL )
+  {
+    panicbug("Couldn't allocate bitmap: %s", SDL_GetError());
+    return(NULL);
+  }
+
+  srcpitch = ((w + 7) / 8);
+  dstbits = (Uint8 *)bitmap->pixels;
+  int mask = 1;
+
+  /* Copy the pixels */
+  for (y = 0 ; y < h ; y++)
+  {
+    for (x = 0 ; x < w ; x++)
+    {
+      dstbits[x] = (srcbits[x / 8] & mask) ? 1 : 0;
+      mask <<= 1;
+      mask |= (mask >> 8);
+      mask &= 0xFF;
+    }
+    dstbits += bitmap->pitch;
+    srcbits += srcpitch;
+  }
+
+  return(bitmap);
 }
 
 /*-----------------------------------------------------------------------*/
@@ -84,13 +105,21 @@ static SDL_Surface *SDLGui_LoadXBM(int w, int h, uint8 *bits)
 bool SDLGui_Init()
 {
   /* Load the font graphics: */
-  stdfontgfx = SDLGui_LoadXBM(font8_width, font8_height, font8_bits);
-  if( stdfontgfx==NULL )
+  fontgfx = SDLGui_LoadXBM(font8_width, font8_height, font8_bits);
+  if (fontgfx == NULL)
   {
     panicbug("Could not create font data");
     panicbug("ARAnyM GUI will not be available");
     return false;
   }
+
+  /* Set font color 0 as transparent */
+  SDL_SetColorKey(fontgfx, SDL_SRCCOLORKEY, 0);
+
+  /* Get the font width and height: */
+  fontwidth = fontgfx->w/16;
+  fontheight = fontgfx->h/16;
+
   return true;
 }
 
@@ -101,11 +130,8 @@ bool SDLGui_Init()
 */
 int SDLGui_UnInit()
 {
-  if(stdfontgfx) {
-    SDL_FreeSurface(stdfontgfx);
-    stdfontgfx = NULL;
-  }
-  if(fontgfx) {
+  if (fontgfx)
+  {
     SDL_FreeSurface(fontgfx);
     fontgfx = NULL;
   }
@@ -114,55 +140,6 @@ int SDLGui_UnInit()
 }
 
 
-/*-----------------------------------------------------------------------*/
-/*
-  Prepare the font to suit the actual resolution.
-*/
-int SDLGui_PrepareFont()
-{
-  if( stdfontgfx == NULL )
-  {
-    panicbug("Error: The font has not been loaded!");
-    return -1;
-  }
-
-  /* Convert the font graphics to the actual screen format */
-  SCRLOCK;
-  fontgfx = SDL_DisplayFormat(stdfontgfx);
-  SCRUNLOCK;
-  if( fontgfx==NULL )
-  {
-    panicbug("Could not convert font: %s", SDL_GetError() );
-    return -1;
-  }
-  /* Set transparent pixel */
-  SCRLOCK;
-  SDL_SetColorKey(fontgfx, (SDL_SRCCOLORKEY|SDL_RLEACCEL), SDL_MapRGB(fontgfx->format,255,255,255));
-  SCRUNLOCK;
-  /* Get the font width and height: */
-  fontwidth = fontgfx->w/16;
-  fontheight = fontgfx->h/16;
-
-  return 0;
-}
-
-/*-----------------------------------------------------------------------*/
-/*
-  Free the font.
-*/
-void SDLGui_FreeFont()
-{
-  if(fontgfx) {
-    SDL_FreeSurface(fontgfx);
-    fontgfx = NULL;
-  }
-
-  // No dialog on screen anymore -> update stored coordinates
-  DialogRect.w = 0;
-  DialogRect.y = 0;
-}
-
-
 /*-----------------------------------------------------------------------*/
 /*
   Compute real coordinates for a given object.
@@ -219,6 +196,8 @@ void SDLGui_ObjFullCoord(SGOBJ *dlg, int objnum, SDL_Rect *coord)
       }
       break;
     case SGEDITFIELD:
+      // Allow one more pixel to the right for cursor
+      coord->w += 1;
       // There is a line below
       coord->h += 1;
       break;
@@ -263,6 +242,16 @@ void SDLGui_UpdateRect(SDL_Rect *rect)
 }
 
 
+/*-----------------------------------------------------------------------*/
+/*
+  Maps an SDL_Color to the screen format.
+*/
+Uint32 SDLGui_MapColor(SDL_Color *color)
+{
+  return SDL_MapRGB(sdlscrn->format, color->r, color->g, color->b);
+}
+
+
 /*-----------------------------------------------------------------------*/
 /*
   Refresh display to reflect an object change.
@@ -283,12 +272,14 @@ void SDLGui_RefreshObj(SGOBJ *dlg, int objnum)
 /*
   Draw a text string.
 */
-void SDLGui_Text(int x, int y, const char *txt)
+void SDLGui_Text(int x, int y, const char *txt, SDL_Color *col)
 {
   int i;
   char c;
   SDL_Rect sr, dr;
 
+  SDL_SetColors(fontgfx, col, 1, 1);
+
   SCRLOCK;
   for (i = 0 ; txt[i] != 0 ; i++)
   {
@@ -316,10 +307,27 @@ void SDLGui_Text(int x, int y, const char *txt)
 void SDLGui_DrawText(SGOBJ *tdlg, int objnum)
 {
   SDL_Rect coord;
+  SDL_Color *textc, *backgroundc;
+
+  if (tdlg[objnum].state & SG_SELECTED)
+  {
+    textc       = whitec;
+    backgroundc = darkgreyc;
+  }
+  else if (tdlg[objnum].state & SG_DISABLED)
+  {
+    textc       = darkgreyc;
+    backgroundc = greyc;
+  }
+  else
+  {
+    textc       = blackc;
+    backgroundc = greyc;
+  }
   
   SDLGui_ObjCoord(tdlg, objnum, &coord);
-  SDL_FillRect(sdlscrn, &coord, SDL_MapRGB(sdlscrn->format,192,192,192));
-  SDLGui_Text(coord.x, coord.y, tdlg[objnum].txt);
+  SDL_FillRect(sdlscrn, &coord, SDLGui_MapColor(backgroundc));
+  SDLGui_Text(coord.x, coord.y, tdlg[objnum].txt, textc);
 }
 
 
@@ -330,15 +338,50 @@ void SDLGui_DrawText(SGOBJ *tdlg, int objnum)
 void SDLGui_DrawEditField(SGOBJ *edlg, int objnum)
 {
   SDL_Rect coord;
+  SDL_Color *textc;
+
+  if (edlg[objnum].state & SG_DISABLED)
+    textc = darkgreyc;
+  else
+    textc = blackc;
 
   SDLGui_ObjCoord(edlg, objnum, &coord);
-  SDL_FillRect(sdlscrn, &coord, SDL_MapRGB(sdlscrn->format,192,192,192));
-  SDLGui_Text(coord.x, coord.y, edlg[objnum].txt);
+  coord.w += 1;
+  SDL_FillRect(sdlscrn, &coord, SDLGui_MapColor(greyc));
+  SDLGui_Text(coord.x, coord.y, edlg[objnum].txt, textc);
 
   // Draw a line below.
   coord.y = coord.y + coord.h;
   coord.h = 1;
-  SDL_FillRect(sdlscrn, &coord, SDL_MapRGB(sdlscrn->format,128,128,128));
+  SDL_FillRect(sdlscrn, &coord, SDLGui_MapColor(darkgreyc));
+}
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Draw or erase cursor.
+*/
+void SDLGui_DrawCursor(SGOBJ *dlg, cursor_state *cursor)
+{
+  if (cursor->object != -1)
+  {
+    SDL_Rect coord;
+    SDL_Color *cursorc;
+
+    SDLGui_DrawEditField(dlg, cursor->object);
+
+    if (cursor->blink_state)
+      cursorc = blackc;
+    else
+      cursorc = greyc;
+
+    SDLGui_ObjCoord(dlg, cursor->object, &coord);
+    coord.x += (cursor->position * fontwidth);
+    coord.w = 1;
+    SDL_FillRect(sdlscrn, &coord, SDLGui_MapColor(cursorc));
+
+    SDLGui_RefreshObj(dlg, cursor->object);
+  }
 }
 
 
@@ -347,10 +390,13 @@ void SDLGui_DrawEditField(SGOBJ *edlg, int objnum)
   Draw a 3D effect around a given rectangle.
   Rectangle is updated to the full size of the new object.
 */
-void SDLGui_Draw3DAround(SDL_Rect *coord, Uint32 upleftc, Uint32 downrightc, Uint32 cornerc, int width)
+void SDLGui_Draw3DAround(SDL_Rect *coord, SDL_Color *upleftc, SDL_Color *downrightc, SDL_Color *cornerc, int width)
 {
   SDL_Rect rect;
   int i;
+  Uint32 upleftcol    = SDLGui_MapColor(upleftc);
+  Uint32 downrightcol = SDLGui_MapColor(downrightc);
+  Uint32 cornercol    = SDLGui_MapColor(cornerc);
 
   SCRLOCK;
 
@@ -360,37 +406,37 @@ void SDLGui_Draw3DAround(SDL_Rect *coord, Uint32 upleftc, Uint32 downrightc, Uin
     rect.y = coord->y - i;
     rect.w = coord->w + (i * 2) - 1;
     rect.h = 1;
-    SDL_FillRect(sdlscrn, &rect, upleftc);
+    SDL_FillRect(sdlscrn, &rect, upleftcol);
 
     rect.x = coord->x - i;
     rect.y = coord->y - i;
     rect.w = 1;
     rect.h = coord->h + (i * 2) - 1;
-    SDL_FillRect(sdlscrn, &rect, upleftc);
+    SDL_FillRect(sdlscrn, &rect, upleftcol);
 
     rect.x = coord->x - i + 1;
     rect.y = coord->y + coord->h - 1 + i;
     rect.w = coord->w + (i * 2) - 1;
     rect.h = 1;
-    SDL_FillRect(sdlscrn, &rect, downrightc);
+    SDL_FillRect(sdlscrn, &rect, downrightcol);
 
     rect.x = coord->x + coord->w - 1 + i;
     rect.y = coord->y - i + 1;
     rect.w = 1;
     rect.h = coord->h + (i * 2) - 1;
-    SDL_FillRect(sdlscrn, &rect, downrightc);
+    SDL_FillRect(sdlscrn, &rect, downrightcol);
 
     rect.x = coord->x + coord->w + i - 1;
     rect.y = coord->y - i;
     rect.w = 1;
     rect.h = 1;
-    SDL_FillRect(sdlscrn, &rect, cornerc);
+    SDL_FillRect(sdlscrn, &rect, cornercol);
 
     rect.x = coord->x - i;
     rect.y = coord->y + coord->h + i - 1;
     rect.w = 1;
     rect.h = 1;
-    SDL_FillRect(sdlscrn, &rect, cornerc);
+    SDL_FillRect(sdlscrn, &rect, cornercol);
   }
 
   SCRUNLOCK;
@@ -407,9 +453,10 @@ void SDLGui_Draw3DAround(SDL_Rect *coord, Uint32 upleftc, Uint32 downrightc, Uin
   Draw a colored box around a given rectangle.
   Rectangle is updated to the full size of the new object.
 */
-void SDLGui_DrawBoxAround(SDL_Rect *coord, Uint32 color, int width)
+void SDLGui_DrawBoxAround(SDL_Rect *coord, SDL_Color *color, int width)
 {
   SDL_Rect rect;
+  Uint32 col = SDLGui_MapColor(color);
 
   SCRLOCK;
 
@@ -417,25 +464,25 @@ void SDLGui_DrawBoxAround(SDL_Rect *coord, Uint32 color, int width)
   rect.y = coord->y - width;
   rect.w = coord->w + (width * 2);
   rect.h = width;
-  SDL_FillRect(sdlscrn, &rect, color);
+  SDL_FillRect(sdlscrn, &rect, col);
 
   rect.x = coord->x - width;
   rect.y = coord->y - width;
   rect.w = width;
   rect.h = coord->h + (width * 2);
-  SDL_FillRect(sdlscrn, &rect, color);
+  SDL_FillRect(sdlscrn, &rect, col);
 
   rect.x = coord->x + coord->w;
   rect.y = coord->y - width;
   rect.w = width;
   rect.h = coord->h + (width * 2);
-  SDL_FillRect(sdlscrn, &rect, color);
+  SDL_FillRect(sdlscrn, &rect, col);
 
   rect.x = coord->x - width;
   rect.y = coord->y + coord->h;
   rect.w = coord->w + (width * 2);
   rect.h = width;
-  SDL_FillRect(sdlscrn, &rect, color);
+  SDL_FillRect(sdlscrn, &rect, col);
 
   SCRUNLOCK;
 
@@ -451,11 +498,11 @@ void SDLGui_DrawBoxAround(SDL_Rect *coord, Uint32 color, int width)
   Draw a 3D box with given attributes.
 */
 void SDLGui_Draw3DBox(SDL_Rect *coord,
-                      Uint32 backgroundc,
-                      Uint32 inboxc,
-                      Uint32 upleftc,
-                      Uint32 downrightc,
-                      Uint32 outboxc,
+                      SDL_Color *backgroundc,
+                      SDL_Color *inboxc,
+                      SDL_Color *upleftc,
+                      SDL_Color *downrightc,
+                      SDL_Color *outboxc,
                       int widthbackground,
                       int widthinbox,
                       int width3D1,
@@ -471,7 +518,7 @@ void SDLGui_Draw3DBox(SDL_Rect *coord,
   rect.y = coord->y - widthbackground;
   rect.w = coord->w + (widthbackground * 2);
   rect.h = coord->h + (widthbackground * 2);
-  SDL_FillRect(sdlscrn, &rect, backgroundc);
+  SDL_FillRect(sdlscrn, &rect, SDLGui_MapColor(backgroundc));
 
   SCRUNLOCK;
 
@@ -502,15 +549,17 @@ void SDLGui_Draw3DBox(SDL_Rect *coord,
 void SDLGui_DrawBox(SGOBJ *bdlg, int objnum)
 {
   SDL_Rect coord;
-  Uint32 upleftc, downrightc;
-  Uint32 darkgreyc = SDL_MapRGB(sdlscrn->format,128,128,128);
-  Uint32 whitec    = SDL_MapRGB(sdlscrn->format,255,255,255);
-  Uint32 greyc     = SDL_MapRGB(sdlscrn->format,192,192,192);
-  Uint32 blackc    = SDL_MapRGB(sdlscrn->format,0,0,0);
+  SDL_Color *my_blackc;
+  SDL_Color *upleftc, *downrightc;
 
   SDLGui_ObjCoord(bdlg, objnum, &coord);
 
   // Modify box drawing according to object state
+  if (bdlg[objnum].state & SG_DISABLED)
+    my_blackc = darkgreyc;
+  else
+    my_blackc = blackc;
+
   if (bdlg[objnum].state & SG_SELECTED)
   {
     upleftc    = darkgreyc;
@@ -528,25 +577,25 @@ void SDLGui_DrawBox(SGOBJ *bdlg, int objnum)
     case (SG_SELECTABLE | SG_DEFAULT | SG_BACKGROUND):
     case (SG_SELECTABLE | SG_DEFAULT):
       SDLGui_Draw3DBox(&coord,
-                       greyc, 0, upleftc, downrightc, blackc,
+                       greyc, NULL, upleftc, downrightc, my_blackc,
                        1, 0, 1, 0, 2);
       break;
     case (SG_SELECTABLE | SG_BACKGROUND):
     case SG_SELECTABLE:
       SDLGui_Draw3DBox(&coord,
-                       greyc, 0, upleftc, downrightc, blackc,
+                       greyc, NULL, upleftc, downrightc, my_blackc,
                        1, 0, 1, 0, 1);
       break;
     case (SG_DEFAULT | SG_BACKGROUND):
     case SG_BACKGROUND:
       SDLGui_Draw3DBox(&coord,
-                       greyc, blackc, upleftc, downrightc, darkgreyc,
+                       greyc, my_blackc, upleftc, downrightc, darkgreyc,
                        0, 2, 3, 0, 1);
       break;
     case SG_DEFAULT:
     case 0:
       SDLGui_Draw3DBox(&coord,
-                       greyc, 0, upleftc, downrightc, 0,
+                       greyc, NULL, upleftc, downrightc, NULL,
                        3, 0, 1, 1, 0);
       break;
   }
@@ -560,7 +609,8 @@ void SDLGui_DrawBox(SGOBJ *bdlg, int objnum)
 void SDLGui_DrawButton(SGOBJ *bdlg, int objnum)
 {
   SDL_Rect coord;
-  int x,y;
+  int x, y;
+  SDL_Color *textc;
 
   SDLGui_ObjCoord(bdlg, objnum, &coord);
 
@@ -573,54 +623,13 @@ void SDLGui_DrawButton(SGOBJ *bdlg, int objnum)
     y += 1;
   }
 
-  SDLGui_DrawBox(bdlg, objnum);
-  SDLGui_Text(x, y, bdlg[objnum].txt);
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Draw a dialog radio button object state.
-*/
-void SDLGui_DrawRadioButtonState(SGOBJ *rdlg, int objnum)
-{
-  Uint32 grey = SDL_MapRGB(sdlscrn->format,192,192,192);
-  SDL_Rect coord;
-  char str[2];
-
-  SDLGui_ObjCoord(rdlg, objnum, &coord);
-
-  if (rdlg[objnum].state & SG_SELECTED)
-  {
-    str[0]=SGRADIOBUTTON_SELECTED;
-  }
+  if (bdlg[objnum].state & SG_DISABLED)
+    textc = darkgreyc;
   else
-  {
-    str[0]=SGRADIOBUTTON_NORMAL;
-  }
-  str[1]='\0';
+    textc = blackc;
 
-  coord.w = fontwidth;
-  coord.h = fontheight;
-  SDL_FillRect(sdlscrn, &coord, grey);
-  SDLGui_Text(coord.x, coord.y, str);
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Draw a dialog radio button object.
-*/
-void SDLGui_DrawRadioButton(SGOBJ *rdlg, int objnum)
-{
-  SDL_Rect coord;
-
-  SDLGui_ObjCoord(rdlg, objnum, &coord);
-
-  coord.x += (fontwidth * 2);
-  SDLGui_Text(coord.x, coord.y, rdlg[objnum].txt);
-
-  SDLGui_DrawRadioButtonState(rdlg, objnum);
+  SDLGui_DrawBox(bdlg, objnum);
+  SDLGui_Text(x, y, bdlg[objnum].txt, textc);
 }
 
 
@@ -630,20 +639,33 @@ void SDLGui_DrawRadioButton(SGOBJ *rdlg, int objnum)
 */
 void SDLGui_DrawCheckBoxState(SGOBJ *cdlg, int objnum)
 {
-  Uint32 grey = SDL_MapRGB(sdlscrn->format,192,192,192);
+  Uint32 grey = SDLGui_MapColor(greyc);
   SDL_Rect coord;
   char str[2];
+  SDL_Color *textc;
 
   SDLGui_ObjCoord(cdlg, objnum, &coord);
 
-  if (cdlg[objnum].state & SG_SELECTED)
+  if (cdlg[objnum].flags & SG_RADIO)
   {
-    str[0]=SGCHECKBOX_SELECTED;
+    if (cdlg[objnum].state & SG_SELECTED)
+      str[0]=SGCHECKBOX_RADIO_SELECTED;
+    else
+      str[0]=SGCHECKBOX_RADIO_NORMAL;
   }
   else
   {
-    str[0]=SGCHECKBOX_NORMAL;
+    if (cdlg[objnum].state & SG_SELECTED)
+      str[0]=SGCHECKBOX_SELECTED;
+    else
+      str[0]=SGCHECKBOX_NORMAL;
   }
+
+  if (cdlg[objnum].state & SG_DISABLED)
+    textc = darkgreyc;
+  else
+    textc = blackc;
+
   str[1]='\0';
 
   coord.w = fontwidth;
@@ -653,7 +675,7 @@ void SDLGui_DrawCheckBoxState(SGOBJ *cdlg, int objnum)
     coord.x += ((strlen(cdlg[objnum].txt) + 1) * fontwidth);
 
   SDL_FillRect(sdlscrn, &coord, grey);
-  SDLGui_Text(coord.x, coord.y, str);
+  SDLGui_Text(coord.x, coord.y, str, textc);
 }
 
 
@@ -664,13 +686,19 @@ void SDLGui_DrawCheckBoxState(SGOBJ *cdlg, int objnum)
 void SDLGui_DrawCheckBox(SGOBJ *cdlg, int objnum)
 {
   SDL_Rect coord;
+  SDL_Color *textc;
 
   SDLGui_ObjCoord(cdlg, objnum, &coord);
 
   if (!(cdlg[objnum].flags&SG_BUTTON_RIGHT))
     coord.x += (fontwidth * 2);
 
-  SDLGui_Text(coord.x, coord.y, cdlg[objnum].txt);
+  if (cdlg[objnum].state & SG_DISABLED)
+    textc = darkgreyc;
+  else
+    textc = blackc;
+
+  SDLGui_Text(coord.x, coord.y, cdlg[objnum].txt, textc);
   SDLGui_DrawCheckBoxState(cdlg, objnum);
 }
 
@@ -683,126 +711,19 @@ void SDLGui_DrawPopupButton(SGOBJ *pdlg, int objnum)
 {
   SDL_Rect coord;
   const char *downstr = "\x02";
+  SDL_Color *textc;
+
+  if (pdlg[objnum].state & SG_DISABLED)
+    textc = darkgreyc;
+  else
+    textc = blackc;
 
   SDLGui_DrawBox(pdlg, objnum);
 
   SDLGui_ObjCoord(pdlg, objnum, &coord);
 
-  SDLGui_Text(coord.x, coord.y, pdlg[objnum].txt);
-  SDLGui_Text(coord.x+coord.w-fontwidth, coord.y, downstr);
-}
-
-
-/*-----------------------------------------------------------------------*/
-/*
-  Let the user insert text into an edit field object.
-  NOTE: The dlg[objnum].txt must point to an an array that is big enough
-  for dlg[objnum].w characters!
-*/
-void SDLGui_EditField(SGOBJ *dlg, int objnum)
-{
-  unsigned int cursorPos;               /* Position of the cursor in the edit field */
-  int blinkState = 0;                   /* Used for cursor blinking */
-  bool bStopEditing = false;            /* true if user wants to exit the edit field */
-  char *txt;                            /* Shortcut for dlg[objnum].txt */
-  SDL_Rect rect;
-  Uint32 grey, cursorCol;
-  SDL_Event event;
-
-  grey = SDL_MapRGB(sdlscrn->format,192,192,192);
-  cursorCol = SDL_MapRGB(sdlscrn->format,128,128,128);
-
-  SDLGui_ObjCoord(dlg, objnum, &rect);
-  // Add some place for cursor
-  rect.w += fontwidth;
-
-  txt = dlg[objnum].txt;
-  cursorPos = strlen(txt);
-
-  do
-  {
-    /* Look for events */
-    if (SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_EVENTMASK(SDL_USEREVENT)) == 0)
-    {
-      /* No event: Wait some time for cursor blinking */
-      SDL_Delay(250);
-      blinkState ^= 1;
-    }
-    else
-    {
-      /* Handle events */
-      do
-      {
-        switch(event.user.code)
-        {
-          case SDL_MOUSEBUTTONDOWN:             /* Mouse pressed -> stop editing */
-            bStopEditing = true;
-            break;
-          case SDL_KEYDOWN:                     /* Key pressed */
-            int keysym = (int)event.user.data1;
-            switch(keysym)
-            {
-              case SDLK_RETURN:
-              case SDLK_KP_ENTER:
-                bStopEditing = true;
-                break;
-              case SDLK_LEFT:
-                if(cursorPos > 0)
-                  cursorPos -= 1;
-                break;
-              case SDLK_RIGHT:
-                if(cursorPos < strlen(txt))
-                  cursorPos += 1;
-                break;
-              case SDLK_BACKSPACE:
-                if(cursorPos > 0)
-                {
-                  memmove(&txt[cursorPos-1], &txt[cursorPos], strlen(&txt[cursorPos])+1);
-                  cursorPos -= 1;
-                }
-                break;
-              case SDLK_DELETE:
-                if(cursorPos < strlen(txt))
-                  memmove(&txt[cursorPos], &txt[cursorPos+1], strlen(&txt[cursorPos+1])+1);
-                break;
-              default:
-                /* If it is a "good" key then insert it into the text field */
-                if(keysym >= 32 && keysym < 256)
-                {
-                  if(strlen(txt) < dlg[objnum].w)
-                  {
-                    memmove(&txt[cursorPos+1], &txt[cursorPos], strlen(&txt[cursorPos])+1);
-                    if(event.key.keysym.mod & (KMOD_LSHIFT|KMOD_RSHIFT))
-                      txt[cursorPos] = toupper(keysym);
-                    else
-                      txt[cursorPos] = keysym;
-                    cursorPos += 1;
-                  }
-                }
-                break;
-            }
-            break;
-        }
-      }
-      while(SDL_PeepEvents(&event, 1, SDL_GETEVENT, SDL_EVENTMASK(SDL_USEREVENT))>0);
-
-      blinkState = 1;
-    }
-
-    /* Redraw the text field: */
-    SDL_FillRect(sdlscrn, &rect, grey);  /* Draw background */
-    /* Draw the cursor: */
-    if(blinkState && !bStopEditing)
-    {
-      SDL_Rect cursorrect;
-      cursorrect.x = rect.x + cursorPos * fontwidth;  cursorrect.y = rect.y;
-      cursorrect.w = fontwidth;  cursorrect.h = rect.h;
-      SDL_FillRect(sdlscrn, &cursorrect, cursorCol);
-    }
-    SDLGui_Text(rect.x, rect.y, dlg[objnum].txt);  /* Draw text */
-    SDLGui_RefreshObj(dlg, objnum);
-  }
-  while(!bStopEditing);
+  SDLGui_Text(coord.x, coord.y, pdlg[objnum].txt, textc);
+  SDLGui_Text(coord.x+coord.w-fontwidth, coord.y, downstr, textc);
 }
 
 
@@ -812,7 +733,7 @@ void SDLGui_EditField(SGOBJ *dlg, int objnum)
 */
 void SDLGui_DrawObject(SGOBJ *dlg, int objnum)
 {
-  switch( dlg[objnum].type )
+  switch (dlg[objnum].type)
   {
     case SGBOX:
       SDLGui_DrawBox(dlg, objnum);
@@ -826,9 +747,6 @@ void SDLGui_DrawObject(SGOBJ *dlg, int objnum)
     case SGBUTTON:
       SDLGui_DrawButton(dlg, objnum);
       break;
-    case SGRADIOBUT:
-      SDLGui_DrawRadioButton(dlg, objnum);
-      break;
     case SGCHECKBOX:
       SDLGui_DrawCheckBox(dlg, objnum);
       break;
@@ -903,7 +821,7 @@ int SDLGui_FindObj(SGOBJ *dlg, int fx, int fy)
        fx < (coord.x + coord.w) &&
        fy < (coord.y + coord.h))
     {
-      if (dlg[i].state & SG_HIDDEN) continue;
+      if (dlg[i].state & (SG_HIDDEN | SG_DISABLED)) continue;
       ob = i;
       break;
     }
@@ -915,27 +833,26 @@ int SDLGui_FindObj(SGOBJ *dlg, int fx, int fy)
 
 /*-----------------------------------------------------------------------*/
 /*
-  A radio button has been selected. Let's deselect any other in his group.
+  A radio object has been selected. Let's deselect any other in his group.
 */
-void SDLGui_SelectRadioButton(SGOBJ *dlg, int clicked_obj)
+void SDLGui_SelectRadioObject(SGOBJ *dlg, int clicked_obj)
 {
   int obj;
 
-  // Find first radio button in this group
+  // Find first radio object in this group
   obj = clicked_obj;
-  while (dlg[--obj].type == SGRADIOBUT);
+  while (dlg[--obj].flags & SG_RADIO);
 
   // Update state
-  while (dlg[++obj].type == SGRADIOBUT)
+  while (dlg[++obj].flags & SG_RADIO)
   {
     // This code scan every object in the group. This allows to solve cases
-    // where multiple radio-buttons where selected in the group by clicking
-    // one.
+    // where multiple objects where selected in the group by clicking one.
     if ((obj != clicked_obj) && (dlg[obj].state & SG_SELECTED))
     {
       // Deselect this radio button
       dlg[obj].state &= ~SG_SELECTED;
-      SDLGui_DrawRadioButtonState(dlg, obj);
+      SDLGui_DrawObject(dlg, obj);
       SDLGui_RefreshObj(dlg, obj);
     }
   }
@@ -954,11 +871,11 @@ bool SDLGui_UpdateObjState(SGOBJ *dlg, int clicked_obj, int original_state,
 
   obj = SDLGui_FindObj(dlg, x, y);
 
-  // Special case : user clicked on an already selected radio button
+  // Special case : user clicked on an already selected radio object
   // do not modify its state.
   // We handle it here because it allows to exit if the object is SG_EXIT or
   // SG_TOUCHEXIT without any additional test.
-  if ((dlg[clicked_obj].type == SGRADIOBUT) && (original_state & SG_SELECTED))
+  if ((dlg[clicked_obj].flags & SG_RADIO) && (original_state & SG_SELECTED))
     return (obj == clicked_obj);
 
   if (((obj != clicked_obj) &&
@@ -966,27 +883,149 @@ bool SDLGui_UpdateObjState(SGOBJ *dlg, int clicked_obj, int original_state,
       ((obj == clicked_obj) &&
        (dlg[clicked_obj].state == original_state)))
   {
-    switch (dlg[clicked_obj].type)
+    if (dlg[clicked_obj].flags & SG_SELECTABLE)
     {
-      case SGBUTTON:
-      case SGCHECKBOX:
-      case SGPOPUP:
-      case SGRADIOBUT:
-        dlg[clicked_obj].state ^= SG_SELECTED;
-        SDLGui_DrawObject(dlg, clicked_obj);
-        SDLGui_RefreshObj(dlg, clicked_obj);
-        break;
+      dlg[clicked_obj].state ^= SG_SELECTED;
+      SDLGui_DrawObject(dlg, clicked_obj);
+      SDLGui_RefreshObj(dlg, clicked_obj);
     }
   }
 
   return (obj == clicked_obj);
 }
 
+
+/*-----------------------------------------------------------------------*/
+/*
+  Search edit field in a dialog.
+*/
+int SDLGui_FindEditField(SGOBJ *dlg, int objnum, int mode)
+{
+  int i, j;
+
+  switch (mode)
+  {
+    case SG_FIRST_EDITFIELD:
+      i = 0;
+      while (dlg[i].type != -1)
+      {
+        if ((dlg[i].type == SGEDITFIELD) &&
+            ((dlg[i].state & (SG_HIDDEN | SG_DISABLED)) == 0))
+          return i;
+        i++;
+      }
+      break;
+
+    case SG_PREVIOUS_EDITFIELD:
+      i = objnum - 1;
+      while (i >= 0)
+      {
+        if ((dlg[i].type == SGEDITFIELD) &&
+            ((dlg[i].state & (SG_HIDDEN | SG_DISABLED)) == 0))
+          return i;
+        i--;
+      }
+      break;
+
+    case SG_NEXT_EDITFIELD:
+      i = objnum + 1;
+      while (dlg[i].type != -1)
+      {
+        if ((dlg[i].type == SGEDITFIELD) &&
+            ((dlg[i].state & (SG_HIDDEN | SG_DISABLED)) == 0))
+          return i;
+        i++;
+      }
+      break;
+
+    case SG_LAST_EDITFIELD:
+      i = objnum + 1;
+      j = -1;
+      while (dlg[i].type != -1)
+      {
+        if ((dlg[i].type == SGEDITFIELD) &&
+            ((dlg[i].state & (SG_HIDDEN | SG_DISABLED)) == 0))
+          j = i;
+        i++;
+      }
+      if (j != -1)
+        return j;
+      break;
+  }
+
+  return objnum;
+}
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Move cursor to another edit field.
+*/
+void SDLGui_MoveCursor(SGOBJ *dlg, cursor_state *cursor, int mode)
+{
+  int new_object;
+
+  new_object = SDLGui_FindEditField(dlg, cursor->object, mode);
+
+  if (new_object != cursor->object)
+  {
+    /* Erase old cursor */
+    cursor->blink_state = false;
+    SDLGui_DrawCursor(dlg, cursor);
+
+    cursor->object = new_object;
+    cursor->position = strlen(dlg[new_object].txt);
+  }
+  else
+  {
+    /* We stay in the same field */
+    /* Move cursor to begin or end of text depending on mode */
+    switch (mode)
+    {
+      case SG_FIRST_EDITFIELD:
+      case SG_PREVIOUS_EDITFIELD:
+        cursor->position = 0;
+        break;
+
+      case SG_NEXT_EDITFIELD:
+      case SG_LAST_EDITFIELD:
+        cursor->position = strlen(dlg[new_object].txt);
+        break;
+    }
+  }
+}
+
+
+/*-----------------------------------------------------------------------*/
+/*
+  Handle mouse clicks on edit fields.
+*/
+void SDLGui_ClickEditField(SGOBJ *dlg, cursor_state *cursor, int clicked_obj, int x)
+{
+  SDL_Rect coord;
+  int i, j;
+
+  /* Erase old cursor */
+  cursor->blink_state = false;
+  SDLGui_DrawCursor(dlg, cursor);
+
+  SDLGui_ObjFullCoord(dlg, clicked_obj, &coord);
+  i = (x - coord.x + (fontwidth / 2)) / fontwidth;
+  j = strlen(dlg[clicked_obj].txt);
+
+  cursor->object = clicked_obj;
+  cursor->position = MIN(i, j);
+  cursor->blink_state = true;
+  cursor->blink_counter = 0;
+  SDLGui_DrawCursor(dlg, cursor);
+}
+
+
 /*-----------------------------------------------------------------------*/
 /*
   Handle mouse clicks.
 */
-int SDLGui_MouseClick(SGOBJ *dlg, int fx, int fy)
+int SDLGui_MouseClick(SGOBJ *dlg, int fx, int fy, cursor_state *cursor)
 {
   int clicked_obj;
   int return_obj = -1;
@@ -1023,7 +1062,7 @@ int SDLGui_MouseClick(SGOBJ *dlg, int fx, int fy)
           // re-draw dialog
           SDLGui_DrawDialog(dlg);
 
-          // Were done. Exit from mouse click handling.
+          // Exit from mouse click handling.
           clicked_obj = -1;
           break;
 
@@ -1040,19 +1079,15 @@ int SDLGui_MouseClick(SGOBJ *dlg, int fx, int fy)
             if (dlg[clicked_obj].flags & SG_EXIT)
               return_obj = clicked_obj;
 
-            switch (dlg[clicked_obj].type)
-            {
-              case SGEDITFIELD:
-                SDLGui_EditField(dlg, clicked_obj);
-                break;
+            // If it's a SG_RADIO object, deselect other objects in his group.
+            if (dlg[clicked_obj].flags & SG_RADIO)
+              SDLGui_SelectRadioObject(dlg, clicked_obj);
 
-              case SGRADIOBUT:
-                SDLGui_SelectRadioButton(dlg, clicked_obj);
-                break;
-            }
+            if (dlg[clicked_obj].type == SGEDITFIELD)
+              SDLGui_ClickEditField(dlg, cursor, clicked_obj, x);
           }
 
-          // Were done. Exit from mouse click handling.
+          // Exit from mouse click handling.
           clicked_obj = -1;
 
           break;
@@ -1078,11 +1113,102 @@ int SDLGui_MouseClick(SGOBJ *dlg, int fx, int fy)
 /*
   Handle key press.
 */
-int SDLGui_KeyPress(SGOBJ *dlg, int keysym)
+int SDLGui_KeyPress(SGOBJ *dlg, int keysym, int mod, cursor_state *cursor)
 {
   int return_obj = -1;
   int obj;
 
+  if (cursor->object != -1)
+  {
+    switch(keysym)
+    {
+      case SDLK_RETURN:
+      case SDLK_KP_ENTER:
+        break;
+
+      case SDLK_BACKSPACE:
+        if (cursor->position > 0)
+        {
+          memmove(&dlg[cursor->object].txt[cursor->position-1],
+                  &dlg[cursor->object].txt[cursor->position],
+                  strlen(&dlg[cursor->object].txt[cursor->position])+1);
+          cursor->position--;
+        }
+        break;
+
+      case SDLK_DELETE:
+        if(cursor->position < (int)strlen(dlg[cursor->object].txt))
+        {
+          memmove(&dlg[cursor->object].txt[cursor->position],
+                  &dlg[cursor->object].txt[cursor->position+1],
+                  strlen(&dlg[cursor->object].txt[cursor->position+1])+1);
+        }
+        break;
+
+      case SDLK_LEFT:
+        if (cursor->position > 0)
+          cursor->position--;
+        break;
+
+      case SDLK_RIGHT:
+        if (cursor->position < (int)strlen(dlg[cursor->object].txt))
+          cursor->position++;
+        break;
+
+      case SDLK_DOWN:
+        SDLGui_MoveCursor(dlg, cursor, SG_NEXT_EDITFIELD);
+        break;
+
+      case SDLK_UP:
+        SDLGui_MoveCursor(dlg, cursor, SG_PREVIOUS_EDITFIELD);
+        break;
+
+      case SDLK_TAB:
+        if (mod & KMOD_SHIFT)
+          SDLGui_MoveCursor(dlg, cursor, SG_PREVIOUS_EDITFIELD);
+        else
+          SDLGui_MoveCursor(dlg, cursor, SG_NEXT_EDITFIELD);
+        break;
+
+      case SDLK_HOME:
+        if (mod & KMOD_CTRL)
+          SDLGui_MoveCursor(dlg, cursor, SG_FIRST_EDITFIELD);
+        else
+          cursor->position = 0;
+        break;
+
+      case SDLK_END:
+        if (mod & KMOD_CTRL)
+          SDLGui_MoveCursor(dlg, cursor, SG_LAST_EDITFIELD);
+        else
+          cursor->position = strlen(dlg[cursor->object].txt);
+        break;
+
+      default:
+        if ((keysym >= SDLK_KP0) && (keysym <= SDLK_KP9))
+        {
+          // map numpad numbers to normal numbers
+          keysym -= (SDLK_KP0 - SDLK_0);
+        }
+        /* If it is a "good" key then insert it into the text field */
+        if ((keysym >= SDLK_SPACE) && (keysym < SDLK_KP0))
+        {
+          if (strlen(dlg[cursor->object].txt) < dlg[cursor->object].w)
+          {
+            memmove(&dlg[cursor->object].txt[cursor->position+1],
+                    &dlg[cursor->object].txt[cursor->position],
+                    strlen(&dlg[cursor->object].txt[cursor->position])+1);
+            if (mod & KMOD_SHIFT)
+              dlg[cursor->object].txt[cursor->position] = toupper(keysym);
+            else
+              dlg[cursor->object].txt[cursor->position] = keysym;
+            cursor->position += 1;
+          }
+        }
+        break;
+    }
+  }
+
   switch(keysym)
   {
     case SDLK_RETURN:
@@ -1102,6 +1228,12 @@ int SDLGui_KeyPress(SGOBJ *dlg, int keysym)
       break;
   }
 
+  // Force cursor display. Should ease text input.
+  cursor->blink_state = true;
+  cursor->blink_counter = 0;
+  // Redraw current edit field...
+  SDLGui_DrawCursor(dlg, cursor);
+
   return return_obj;
 }
 
@@ -1238,7 +1370,8 @@ int SDLGui_DoDialog(SGOBJ *dlg)
   int return_obj = -1;
   int obj;
   int x, y;
-  int keysym;
+  int keysym, mod;
+  cursor_state cursor;
 
   /* Is the left mouse button still pressed? Yes -> Handle TOUCHEXIT objects here */
 
@@ -1254,6 +1387,14 @@ int SDLGui_DoDialog(SGOBJ *dlg)
     return_obj = obj;
   }
 
+  cursor.object = SDLGui_FindEditField(dlg, -1, SG_FIRST_EDITFIELD);
+  if (cursor.object != -1)
+    cursor.position = strlen(dlg[cursor.object].txt);
+  else
+    cursor.position = -1;
+  cursor.blink_counter = 0;
+  cursor.blink_state = true;
+
   SDLGui_DrawDialog(dlg);
 
   /* The main loop */
@@ -1267,7 +1408,8 @@ int SDLGui_DoDialog(SGOBJ *dlg)
       {
       	case SDL_KEYDOWN:
           keysym = (int)evnt.user.data1;
-          return_obj = SDLGui_KeyPress(dlg, keysym);
+          mod = (int)evnt.user.data2;
+          return_obj = SDLGui_KeyPress(dlg, keysym, mod, &cursor);
       	  break;
 
         case SDL_USEREVENT:
@@ -1278,7 +1420,7 @@ int SDLGui_DoDialog(SGOBJ *dlg)
         case SDL_MOUSEBUTTONDOWN:
           x = (int)evnt.user.data1;
           y = (int)evnt.user.data2;
-          return_obj = SDLGui_MouseClick(dlg, x, y);
+          return_obj = SDLGui_MouseClick(dlg, x, y, &cursor);
           break;
       }
     }
@@ -1286,7 +1428,14 @@ int SDLGui_DoDialog(SGOBJ *dlg)
     {
       // No special event occured.
       // Wait a little to avoid eating CPU.
-      SDL_Delay(100);
+      SDL_Delay(50);
+      cursor.blink_counter++;
+      if (cursor.blink_counter >= 10)
+      {
+        cursor.blink_counter = 0;
+        cursor.blink_state = cursor.blink_state ? false : true;
+        SDLGui_DrawCursor(dlg, &cursor);
+      }
     }
   }
 
