@@ -1,7 +1,7 @@
 /*
  * hostscreen.cpp - host video routines
  *
- * Copyright (c) 2001-2003 Standa of ARAnyM developer team (see AUTHORS)
+ * Copyright (c) 2001-2003 STanda of ARAnyM developer team (see AUTHORS)
  *
  * This file is part of the ARAnyM project which builds a new and powerful
  * TOS/FreeMiNT compatible virtual machine running on almost any hardware.
@@ -279,6 +279,11 @@ void HostScreen::searchVideoMode( uint32 *width, uint32 *height, uint32 *bpp )
 
 void HostScreen::setWindowSize( uint32 width, uint32 height, uint32 bpp )
 {
+	if (bx_options.autozoom.fixedsize) {
+		width = bx_options.autozoom.width;
+		height = bx_options.autozoom.height;
+	}
+
 	// Select a correct video mode
 	searchVideoMode(&width, &height, &bpp);	
 
@@ -379,13 +384,13 @@ void HostScreen::setWindowSize( uint32 width, uint32 height, uint32 bpp )
 		if ((width>SdlGlTextureWidth) || (height>SdlGlTextureHeight)) {
 			this->width = width = SdlGlTextureWidth;
 			this->height = height = SdlGlTextureHeight;
-			bx_options.video.autozoom=1;
-			bx_options.video.autozoomint=0;
+			bx_options.autozoom.enabled = true;
+			bx_options.autozoom.integercoefs = false;
 
 			D(bug("gl: autozoom enabled"));
 		} else {
-			bx_options.video.autozoom=0;
-			bx_options.video.autozoomint=0;
+			bx_options.autozoom.enabled = false;
+			bx_options.autozoom.integercoefs = false;
 			D(bug("gl: autozoom disabled"));
 		}
 	}
@@ -511,12 +516,12 @@ void HostScreen::gfxHLineColor ( int16 x1, int16 x2, int16 y, uint16 pattern, ui
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixx)
-						*(uint8*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint8*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixx)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint8*)pixel = fgColor; // STanda
+							*(uint8*)pixel = fgColor;
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixx)
@@ -526,7 +531,7 @@ void HostScreen::gfxHLineColor ( int16 x1, int16 x2, int16 y, uint16 pattern, ui
 				case 4:
 					for (; pixel<pixellast; pixel += pixx)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint8*)pixel = fgColor; // STanda
+							*(uint8*)pixel = fgColor;
 					break;
 			}
 			break;
@@ -535,12 +540,12 @@ void HostScreen::gfxHLineColor ( int16 x1, int16 x2, int16 y, uint16 pattern, ui
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixx)
-						*(uint16*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint16*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixx)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint16*)pixel = fgColor; // STanda
+							*(uint16*)pixel = fgColor;
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixx)
@@ -550,7 +555,7 @@ void HostScreen::gfxHLineColor ( int16 x1, int16 x2, int16 y, uint16 pattern, ui
 				case 4:
 					for (; pixel<pixellast; pixel += pixx)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint16*)pixel = fgColor; // STanda
+							*(uint16*)pixel = fgColor;
 					break;
 			}
 			break;
@@ -584,12 +589,12 @@ void HostScreen::gfxHLineColor ( int16 x1, int16 x2, int16 y, uint16 pattern, ui
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixx)
-						*(uint32*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint32*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixx)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint32*)pixel = fgColor; // STanda
+							*(uint32*)pixel = fgColor;
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixx)
@@ -599,7 +604,7 @@ void HostScreen::gfxHLineColor ( int16 x1, int16 x2, int16 y, uint16 pattern, ui
 				case 4:
 					for (; pixel<pixellast; pixel += pixx)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint32*)pixel = fgColor; // STanda
+							*(uint32*)pixel = fgColor;
 					break;
 			}
 			break;
@@ -646,12 +651,12 @@ void HostScreen::gfxVLineColor( int16 x, int16 y1, int16 y2,
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixy)
-						*(uint8*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint8*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixy)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint8*)pixel = fgColor; // STanda
+							*(uint8*)pixel = fgColor;
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixy)
@@ -661,7 +666,7 @@ void HostScreen::gfxVLineColor( int16 x, int16 y1, int16 y2,
 				case 4:
 					for (; pixel<pixellast; pixel += pixy)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint8*)pixel = fgColor; // STanda
+							*(uint8*)pixel = fgColor;
 					break;
 			}
 			break;
@@ -669,12 +674,12 @@ void HostScreen::gfxVLineColor( int16 x, int16 y1, int16 y2,
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixy)
-						*(uint16*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint16*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixy)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint16*)pixel = fgColor; // STanda
+							*(uint16*)pixel = fgColor;
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixy)
@@ -684,7 +689,7 @@ void HostScreen::gfxVLineColor( int16 x, int16 y1, int16 y2,
 				case 4:
 					for (; pixel<pixellast; pixel += pixy)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint16*)pixel = fgColor; // STanda
+							*(uint16*)pixel = fgColor;
 					break;
 			}
 			break;
@@ -715,12 +720,12 @@ void HostScreen::gfxVLineColor( int16 x, int16 y1, int16 y2,
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixy)
-						*(uint32*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint32*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixy)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint32*)pixel = fgColor; // STanda
+							*(uint32*)pixel = fgColor;
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixy)
@@ -730,7 +735,7 @@ void HostScreen::gfxVLineColor( int16 x, int16 y1, int16 y2,
 				case 4:
 					for (; pixel<pixellast; pixel += pixy)
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint32*)pixel = fgColor; // STanda
+							*(uint32*)pixel = fgColor;
 					break;
 			}
 			break;
@@ -810,7 +815,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 			switch (logOp) {
 				case 1:
 					for (; x < dx; x++, pixel += pixx) {
-						*(uint8*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint8*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 
 						y += dy;
 						if (y >= dx) {
@@ -822,7 +827,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 				case 2:
 					for (; x < dx; x++, pixel += pixx) {
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint8*)pixel = fgColor; // STanda
+							*(uint8*)pixel = fgColor;
 
 						y += dy;
 						if (y >= dx) {
@@ -846,7 +851,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 				case 4:
 					for (; x < dx; x++, pixel += pixx) {
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint8*)pixel = fgColor; // STanda
+							*(uint8*)pixel = fgColor;
 
 						y += dy;
 						if (y >= dx) {
@@ -861,7 +866,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 			switch (logOp) {
 				case 1:
 					for (; x < dx; x++, pixel += pixx) {
-						*(uint16*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint16*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 
 						y += dy;
 						if (y >= dx) {
@@ -873,7 +878,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 				case 2:
 					for (; x < dx; x++, pixel += pixx) {
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint16*)pixel = fgColor; // STanda
+							*(uint16*)pixel = fgColor;
 
 						y += dy;
 						if (y >= dx) {
@@ -897,7 +902,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 				case 4:
 					for (; x < dx; x++, pixel += pixx) {
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint16*)pixel = fgColor; // STanda
+							*(uint16*)pixel = fgColor;
 
 						y += dy;
 						if (y >= dx) {
@@ -912,7 +917,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 			switch (logOp) {
 				case 1:
 					for (; x < dx; x++, pixel += pixx) {
-						putBpp24Pixel( pixel, (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor) ); // STanda
+						putBpp24Pixel( pixel, (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor) );
 
 						y += dy;
 						if (y >= dx) {
@@ -963,7 +968,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 			switch (logOp) {
 				case 1:
 					for (; x < dx; x++, pixel += pixx) {
-						*(uint32*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+						*(uint32*)pixel = (( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 
 						y += dy;
 						if (y >= dx) {
@@ -975,7 +980,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 				case 2:
 					for (; x < dx; x++, pixel += pixx) {
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) != 0 )
-							*(uint32*)pixel = fgColor; // STanda
+							*(uint32*)pixel = fgColor;
 
 						y += dy;
 						if (y >= dx) {
@@ -999,7 +1004,7 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 				case 4:
 					for (; x < dx; x++, pixel += pixx) {
 						if ( ( pattern & ( 1 << ( (ppos++) & 0xf ) )) == 0 )
-							*(uint32*)pixel = fgColor; // STanda
+							*(uint32*)pixel = fgColor;
 
 						y += dy;
 						if (y >= dx) {
@@ -1017,7 +1022,8 @@ void HostScreen::gfxLineColor( int16 x1, int16 y1, int16 x2, int16 y2,
 
 
 /**
- * Derived from the SDL_gfxPrimitives::boxColor(). The colors are in the destination surface format here.
+ * Derived from the SDL_gfxPrimitives::boxColor().
+ * The colors are in the destination surface format here.
  * The trivial cases optimalization removed.
  *
  * @author STanda
@@ -1047,28 +1053,28 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
-							*(uint8*)pixel = (( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+							*(uint8*)pixel = (( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 							pixel += pixx;
 						}
 					}
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 )
-								*(uint8*)pixel = fgColor; // STanda
+								*(uint8*)pixel = fgColor;
 							pixel += pixx;
 						}
 					}
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 )
@@ -1079,11 +1085,11 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 					break;
 				case 4:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) == 0 )
-								*(uint8*)pixel = fgColor; // STanda
+								*(uint8*)pixel = fgColor;
 							pixel += pixx;
 						}
 					}
@@ -1097,28 +1103,28 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
-							*(uint16*)pixel = (( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+							*(uint16*)pixel = (( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 							pixel += pixx;
 						}
 					}
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 )
-								*(uint16*)pixel = fgColor; // STanda
+								*(uint16*)pixel = fgColor;
 							pixel += pixx;
 						}
 					}
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 )
@@ -1129,11 +1135,11 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 					break;
 				case 4:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) == 0 )
-								*(uint16*)pixel = fgColor; // STanda
+								*(uint16*)pixel = fgColor;
 							pixel += pixx;
 						}
 					}
@@ -1145,17 +1151,17 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
-							putBpp24Pixel( pixel, (( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 ) ? fgColor : bgColor) ); // STanda
+							putBpp24Pixel( pixel, (( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 ) ? fgColor : bgColor) );
 							pixel += pixx;
 						}
 					}
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 )
@@ -1166,7 +1172,7 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 )
@@ -1177,7 +1183,7 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 					break;
 				case 4:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) == 0 )
@@ -1193,28 +1199,28 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 			switch (logOp) {
 				case 1:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
-							*(uint32*)pixel = (( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 ) ? fgColor : bgColor); // STanda
+							*(uint32*)pixel = (( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 ) ? fgColor : bgColor);
 							pixel += pixx;
 						}
 					}
 					break;
 				case 2:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 )
-								*(uint32*)pixel = fgColor; // STanda
+								*(uint32*)pixel = fgColor;
 							pixel += pixx;
 						}
 					}
 					break;
 				case 3:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) != 0 )
@@ -1225,11 +1231,11 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 					break;
 				case 4:
 					for (; pixel<pixellast; pixel += pixy) {
-						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff; // STanda
+						uint16 pattern = areaPattern ? areaPattern[ y++ & 0xf ] : 0xffff;
 
 						for (i=0; i<dx; i++) {
 							if ( ( pattern & ( 1 << ( (x+i) & 0xf ) )) == 0 )
-								*(uint32*)pixel = fgColor; // STanda
+								*(uint32*)pixel = fgColor;
 							pixel += pixx;
 						}
 					}
@@ -1242,6 +1248,15 @@ void HostScreen::gfxBoxColorPattern (int16 x, int16 y, int16 w, int16 h,
 
 /*
  * $Log: hostscreen.cpp,v $
+ * Revision 1.48  2004/10/31 23:15:56  pmandin
+ * Do not need to enable autozoom to force screen size
+ *
+ * Revision 1.47  2004/09/10 17:07:25  pmandin
+ * Autozoom: now you can choose a specific host screen size that will remain constant accross Atari screen size changes
+ *
+ * Revision 1.46  2004/04/26 07:24:50  standa
+ * Just comment adjustments.
+ *
  * Revision 1.45  2004/02/07 13:20:42  joy
  * explanation for some unclear trick
  *
