@@ -1,5 +1,5 @@
 /*
- * $Header: /var/repos/aranym/src/ndebug.cpp,v 1.33 2003/12/27 21:02:40 joy Exp $
+ * $Header: /var/repos/aranym/src/ndebug.cpp,v 1.35 2004/07/28 20:10:09 milan Exp $
  *
  * MJ 2001
  */
@@ -77,8 +77,8 @@ static char *strhelp[] = {"Help:\n",
 	" P <number> <value>   set USP(0), ISP(1), MSP(2), VBR(3), IMASK(4), PC(5)\n",
 	" S <number> <value>   set TC(0), SRP(1), URP(2), DTT0(3), DTT1(4),\n",
 	"                      ITT0(5), ITT1(6), CACR(7), CAAR(8)\n",
-	" i                    enable IRQ\n",
-	" I                    disable IRQ\n",
+	" i                    enable/disable IRQ\n",
+	" I                    enable/disable IRQ debugging\n",
 	" g <address>          start execution at the current address or <address>\n",
 	" t <address>          step one instruction at current addres or <address>\n",
 	" z                    step through one instruction - useful for JSR, DBRA etc\n",
@@ -569,9 +569,9 @@ void ndebug::log2phys(FILE *, uaecptr addr) {
 #ifdef FULLMMU
 	if (regs.mmu_enabled) {
 setjmpagain:
-		jmp_buf excep_env_old;
-		memcpy(excep_env_old, excep_env, sizeof(jmp_buf));
-	        int prb = setjmp(excep_env);
+		JMP_BUF excep_env_old;
+		memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+	        int prb = SETJMP(excep_env);
         	if (prb == 2) {
 			bug("Bus error");
 	                goto setjmpagain;
@@ -584,7 +584,7 @@ setjmpagain:
 		}
 		bug("MMU enabled: %08lx -> %08lx",	(unsigned long)addr,
 			(unsigned long)mmu_translate(addr, FC_DATA, 0, m68k_getpc(), sz_long, 0));
-		memcpy(excep_env, excep_env_old, sizeof(jmp_buf));
+		memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 	} else {
 #endif /* FULLMMU */
 		bug("MMU disabled: %08lx", (unsigned long)addr);
@@ -653,10 +653,10 @@ int ndebug::canon(FILE *f, bool wasGrabbed, uaecptr nextpc, uaecptr &nxdis, uaec
 			m68k_reset();
 			break;
 		case 'i':
-			irqindebug = true;
+			irqindebug = !irqindebug; bug("IRQ %s\n", irqindebug ? "enabled" : "disabled"); break;
 			break;
 		case 'I':
-			irqindebug = false;
+			ignore_irq = !ignore_irq; bug("IRQ debugging %s\n", ignore_irq ? "enabled" : "disabled"); break;
 			break;
 		case 'e':
 			dump_traps (f);
@@ -1168,8 +1168,8 @@ void ndebug::nexit() {
 
 void ndebug::dumpmem(FILE *f, volatile uaecptr addr, uaecptr * nxmem, volatile unsigned int lns)
 {
-	jmp_buf excep_env_old;
-	memcpy(excep_env_old, excep_env, sizeof(jmp_buf));
+	JMP_BUF excep_env_old;
+	memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
 	uaecptr a;
 	broken_in = 0;
 	for (; lns-- && !broken_in;) {
@@ -1177,7 +1177,7 @@ void ndebug::dumpmem(FILE *f, volatile uaecptr addr, uaecptr * nxmem, volatile u
 		fprintf(f, "%08lx: \n", (unsigned long)addr);
 		for (i = 0; i < 16; i++) {
 setjmpagain:
-        		if (setjmp(excep_env) != 0) {
+        		if (SETJMP(excep_env) != 0) {
 				fprintf (f, "UA   ");
 				addr += 2;
 				if (++i == 16) break;
@@ -1195,7 +1195,7 @@ setjmpagain:
 		fprintf(f, "\n");
 	}
 	*nxmem = addr;
-	memcpy(excep_env, excep_env_old, sizeof(jmp_buf));
+	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 
 uae_u32 ndebug::readhex (char v, char **c)
@@ -1343,6 +1343,12 @@ void ndebug::showHistory(unsigned int count) {
 
 /*
  * $Log: ndebug.cpp,v $
+ * Revision 1.35  2004/07/28 20:10:09  milan
+ * switch 'I' modified, debugger can ignore interupt mode
+ *
+ * Revision 1.34  2004/07/09 18:41:56  milan
+ * (sig)set/longjmp corrections
+ *
  * Revision 1.33  2003/12/27 21:02:40  joy
  * warning fixed
  *
