@@ -71,12 +71,16 @@ static struct option const long_options[] =
 #endif
   {"floppy", required_argument, 0, 'a'},
   {"resolution", required_argument, 0, 'r'},
+#ifdef DEBUGGER
   {"debug", no_argument, 0, 'D'},
+#endif
   {"fullscreen", no_argument, 0, 'f'},
   {"nomouse", no_argument, 0, 'N'},
   {"refresh", required_argument, 0, 'v'},
   {"monitor", required_argument, 0, 'm'},
+#if HOSTFS_SUPPORT
   {"disk", required_argument, 0, 'd'},
+#endif
   {"help", no_argument, 0, 'h'},
   {"version", no_argument, 0, 'V'},
   {"config", required_argument, 0, 'c'},
@@ -84,7 +88,9 @@ static struct option const long_options[] =
   {"gui", no_argument, 0, 'G'},
   {"swap-ide", no_argument, 0, 'S'},
   {"emutos", no_argument, 0, 'e'},
+#ifdef ENABLE_LILO
   {"lilo", no_argument, 0, 'l'},
+#endif
   {NULL, 0, NULL, 0}
 };
 
@@ -103,6 +109,8 @@ uint32 FastRAMSize;
 static char config_file[512];
 
 #if !defined(XIF_HOST_IP) && !defined(XIF_ATARI_IP) && !defined(XIF_NETMASK)
+# define XIF_TYPE	"ppp"
+# define XIF_TUNNEL	"tap0"
 # define XIF_HOST_IP	"192.168.0.1"
 # define XIF_ATARI_IP	"192.168.0.2"
 # define XIF_NETMASK	"255.255.255.0"
@@ -120,9 +128,9 @@ static bx_atadevice_options_t *diskd = &bx_options.atadevice[0][1];
 /*************************************************************************/
 struct Config_Tag global_conf[]={
 	{ "FastRAM", Int_Tag, &bx_options.fastram, 0, 0},
-	{ "Floppy", String_Tag, bx_options.floppy.path, sizeof(bx_options.floppy.path), 0},
-	{ "TOS", String_Tag, bx_options.tos_path, sizeof(bx_options.tos_path), 0},
-	{ "EmuTOS", String_Tag, bx_options.emutos_path, sizeof(bx_options.emutos_path), 0},
+	{ "Floppy", Path_Tag, bx_options.floppy.path, sizeof(bx_options.floppy.path), 0},
+	{ "TOS", Path_Tag, bx_options.tos_path, sizeof(bx_options.tos_path), 0},
+	{ "EmuTOS", Path_Tag, bx_options.emutos_path, sizeof(bx_options.emutos_path), 0},
 	{ "AutoGrabMouse", Bool_Tag, &bx_options.autoMouseGrab, 0, 0},
 #ifdef ENABLE_EPSLIMITER
 	{ "EpsEnabled", Bool_Tag, &bx_options.cpu.eps_enabled, 0, 0},
@@ -229,8 +237,6 @@ struct Config_Tag video_conf[]={
 	{ "BootColorDepth", Byte_Tag, &bx_options.video.boot_color_depth, 0, 0},
 	{ "VidelRefresh", Byte_Tag, &bx_options.video.refresh, 0, 0},
 	{ "VidelMonitor", Byte_Tag, &bx_options.video.monitor, 0, 0},
-	{ "AutoZoom", Bool_Tag, &bx_options.video.autozoom, 0, 0},
-	{ "AutoZoomInteger", Bool_Tag, &bx_options.video.autozoomint, 0, 0},
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
@@ -239,8 +245,6 @@ void preset_video() {
   bx_options.video.boot_color_depth = -1;	// Boot in color depth
   bx_options.video.monitor = -1;			// preserve default NVRAM monitor
   bx_options.video.refresh = 2;			// 25 Hz update
-  bx_options.video.autozoom = false;
-  bx_options.video.autozoomint = false;
 }
 
 void postload_video() {
@@ -281,7 +285,7 @@ void presave_opengl() {
 	{ "IsCDROM", Bool_Tag, &Disk->isCDROM, 0, 0},	\
 	{ "ByteSwap", Bool_Tag, &Disk->byteswap, 0, 0},	\
 	{ "ReadOnly", Bool_Tag, &Disk->readonly, 0, 0},	\
-	{ "Path", String_Tag, Disk->path, sizeof(Disk->path), 0},	\
+	{ "Path", Path_Tag, Disk->path, sizeof(Disk->path), 0},	\
 	{ "Cylinders", Int_Tag, &Disk->cylinders, 0, 0},	\
 	{ "Heads", Int_Tag, &Disk->heads, 0, 0},	\
 	{ "SectorsPerTrack", Int_Tag, &Disk->spt, 0, 0},	\
@@ -357,7 +361,7 @@ void presave_ide() {
 
 /*************************************************************************/
 #define DISK_CONFIG(Disk)	struct Config_Tag Disk ## _configs[] = {	\
-	{ "Path", String_Tag, bx_options.Disk.path, sizeof(bx_options.Disk.path), 0},	\
+	{ "Path", Path_Tag, bx_options.Disk.path, sizeof(bx_options.Disk.path), 0},	\
 	{ "Present", Bool_Tag, &bx_options.Disk.present, 0, 0},	\
 	{ "PartID", String_Tag, bx_options.Disk.partID, sizeof(bx_options.Disk.partID), 0},	\
 	{ "ByteSwap", Bool_Tag, &bx_options.Disk.byteswap, 0, 0},	\
@@ -385,45 +389,47 @@ void presave_disk() {
 
 /*************************************************************************/
 struct Config_Tag arafs_conf[]={
-	{ "A", String_Tag, &bx_options.aranymfs[0].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "B", String_Tag, &bx_options.aranymfs[1].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "C", String_Tag, &bx_options.aranymfs[2].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "D", String_Tag, &bx_options.aranymfs[3].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "E", String_Tag, &bx_options.aranymfs[4].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "F", String_Tag, &bx_options.aranymfs[5].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "G", String_Tag, &bx_options.aranymfs[6].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "H", String_Tag, &bx_options.aranymfs[7].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "I", String_Tag, &bx_options.aranymfs[8].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "J", String_Tag, &bx_options.aranymfs[9].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "K", String_Tag, &bx_options.aranymfs[10].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "L", String_Tag, &bx_options.aranymfs[11].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "M", String_Tag, &bx_options.aranymfs[12].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "N", String_Tag, &bx_options.aranymfs[13].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "O", String_Tag, &bx_options.aranymfs[14].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "P", String_Tag, &bx_options.aranymfs[15].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "Q", String_Tag, &bx_options.aranymfs[16].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "R", String_Tag, &bx_options.aranymfs[17].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "S", String_Tag, &bx_options.aranymfs[18].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "T", String_Tag, &bx_options.aranymfs[19].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "U", String_Tag, &bx_options.aranymfs[20].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "V", String_Tag, &bx_options.aranymfs[21].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "W", String_Tag, &bx_options.aranymfs[22].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "X", String_Tag, &bx_options.aranymfs[23].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "Y", String_Tag, &bx_options.aranymfs[24].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
-	{ "Z", String_Tag, &bx_options.aranymfs[25].rootPath, sizeof(bx_options.aranymfs[0].rootPath), 0},
+	{ "A", Path_Tag, bx_options.aranymfs[0].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "B", Path_Tag, bx_options.aranymfs[1].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "C", Path_Tag, bx_options.aranymfs[2].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "D", Path_Tag, bx_options.aranymfs[3].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "E", Path_Tag, bx_options.aranymfs[4].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "F", Path_Tag, bx_options.aranymfs[5].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "G", Path_Tag, bx_options.aranymfs[6].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "H", Path_Tag, bx_options.aranymfs[7].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "I", Path_Tag, bx_options.aranymfs[8].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "J", Path_Tag, bx_options.aranymfs[9].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "K", Path_Tag, bx_options.aranymfs[10].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "L", Path_Tag, bx_options.aranymfs[11].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "M", Path_Tag, bx_options.aranymfs[12].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "N", Path_Tag, bx_options.aranymfs[13].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "O", Path_Tag, bx_options.aranymfs[14].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "P", Path_Tag, bx_options.aranymfs[15].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "Q", Path_Tag, bx_options.aranymfs[16].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "R", Path_Tag, bx_options.aranymfs[17].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "S", Path_Tag, bx_options.aranymfs[18].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "T", Path_Tag, bx_options.aranymfs[19].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "U", Path_Tag, bx_options.aranymfs[20].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "V", Path_Tag, bx_options.aranymfs[21].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "W", Path_Tag, bx_options.aranymfs[22].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "X", Path_Tag, bx_options.aranymfs[23].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "Y", Path_Tag, bx_options.aranymfs[24].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
+	{ "Z", Path_Tag, bx_options.aranymfs[25].configPath, sizeof(bx_options.aranymfs[0].configPath), 0},
 	{ NULL , Error_Tag, NULL, 0, 0}
 };
 
 void preset_arafs() {
 	for(int i=0; i < 'Z'-'A'+1; i++) {
 		bx_options.aranymfs[i].rootPath[0] = '\0';
+		bx_options.aranymfs[i].configPath[0] = '\0';
 		bx_options.aranymfs[i].halfSensitive = true;
 	}
 }
 
 void postload_arafs() {
 	for(int i=0; i < 'Z'-'A'+1; i++) {
-		int len = strlen(bx_options.aranymfs[i].rootPath);
+		safe_strncpy(bx_options.aranymfs[i].rootPath, bx_options.aranymfs[i].configPath, sizeof(bx_options.aranymfs[i].rootPath));
+		int len = strlen(bx_options.aranymfs[i].configPath);
 		bx_options.aranymfs[i].halfSensitive = true;
 		if (len > 0) {
 			char *ptrLast = bx_options.aranymfs[i].rootPath + len-1;
@@ -437,11 +443,11 @@ void postload_arafs() {
 
 void presave_arafs() {
 	for(int i=0; i < 'Z'-'A'+1; i++) {
+		safe_strncpy(bx_options.aranymfs[i].configPath, bx_options.aranymfs[i].rootPath, sizeof(bx_options.aranymfs[i].configPath));
 		if ( strlen(bx_options.aranymfs[i].rootPath) > 0 &&
 			 bx_options.aranymfs[i].halfSensitive ) {
 			// set the halfSensitive indicator
-			// NOTE: I can't add more chars here due to the fixed char[] width
-			strcat( bx_options.aranymfs[i].rootPath, ":" );
+			strcat( bx_options.aranymfs[i].configPath, ":" );
 		}
 	}
 }
@@ -449,6 +455,8 @@ void presave_arafs() {
 /*************************************************************************/
 #define ETH(x) bx_options.ethernet.x
 struct Config_Tag ethernet_conf[]={
+	{ "Type", String_Tag, &ETH(type), sizeof(ETH(type)), 0},
+	{ "Tunnel", String_Tag, &ETH(tunnel), sizeof(ETH(tunnel)), 0},
 	{ "HostIP", String_Tag, &ETH(ip_host), sizeof(ETH(ip_host)), 0},
 	{ "AtariIP", String_Tag, &ETH(ip_atari), sizeof(ETH(ip_atari)), 0},
 	{ "Netmask", String_Tag, &ETH(netmask), sizeof(ETH(netmask)), 0},
@@ -456,6 +464,8 @@ struct Config_Tag ethernet_conf[]={
 };
 
 void preset_ethernet() {
+  safe_strncpy(ETH(type), XIF_TYPE, sizeof(ETH(type)));
+  safe_strncpy(ETH(tunnel), XIF_TUNNEL, sizeof(ETH(tunnel)));
   safe_strncpy(ETH(ip_host), XIF_HOST_IP, sizeof(ETH(ip_host)));
   safe_strncpy(ETH(ip_atari ), XIF_ATARI_IP, sizeof(ETH(ip_atari)));
   safe_strncpy(ETH(netmask), XIF_NETMASK, sizeof(ETH(netmask)));
@@ -494,7 +504,7 @@ void presave_lilo() {
 
 struct Config_Tag midi_conf[]={
 	{ "Enabled", Bool_Tag, &MIDI(enabled), 0, 0},
-	{ "Output", String_Tag, &MIDI(output), sizeof(MIDI(output)), 0},
+	{ "Output", Path_Tag, &MIDI(output), sizeof(MIDI(output)), 0},
 	{ NULL , Error_Tag, NULL, 0, 0 }
 };
 
@@ -555,6 +565,30 @@ void postload_nfcdroms() {
 void presave_nfcdroms() {
 }
 
+/*************************************************************************/
+struct Config_Tag autozoom_conf[]={
+	{ "Enabled", Bool_Tag, &bx_options.autozoom.enabled, 0, 0},
+	{ "IntegerCoefs", Bool_Tag, &bx_options.autozoom.integercoefs, 0, 0},
+	{ "FixedSize", Bool_Tag, &bx_options.autozoom.fixedsize, 0, 0},
+	{ "Width", Int_Tag, &bx_options.autozoom.width, 0, 0},
+	{ "Height", Int_Tag, &bx_options.autozoom.height, 0, 0},
+	{ NULL , Error_Tag, NULL, 0, 0 }
+};
+
+void preset_autozoom() {
+  bx_options.autozoom.enabled = false;
+  bx_options.autozoom.integercoefs = false;
+  bx_options.autozoom.fixedsize = false;
+  bx_options.autozoom.width = 640;
+  bx_options.autozoom.height = 480;
+}
+
+void postload_autozoom() {
+}
+
+void presave_autozoom() {
+}
+
 /*************************************************************************/
 void usage (int status) {
   printf ("Usage: %s [OPTIONS]\n", program_name);
@@ -562,13 +596,11 @@ void usage (int status) {
 Options:\n\
   -a, --floppy NAME          floppy image file NAME\n\
   -e, --emutos               boot EmuTOS\n\
-  -l, --lilo                 boot a linux kernel\n\
   -N, --nomouse              don't grab mouse at startup\n\
   -f, --fullscreen           start in fullscreen\n\
   -v, --refresh <X>          VIDEL refresh rate in VBL (default 2)\n\
   -r, --resolution <X>       boot in X color depth [1,2,4,8,16]\n\
   -m, --monitor <X>          attached monitor: 0 = VGA, 1 = TV\n\
-  -d, --disk CHAR:PATH[:]    HostFS mapping, e.g. d:/atari/d_drive\n\
   -c, --config FILE          read different configuration file\n\
   -s, --save                 save configuration file\n\
   -G, --gui                  open GUI at startup\n\
@@ -576,6 +608,12 @@ Options:\n\
   -h, --help                 display this help and exit\n\
   -V, --version              output version information and exit\n\
 ");
+#if HOSTFS_SUPPORT
+  printf("  -d, --disk CHAR:PATH[:]    HostFS mapping, e.g. d:/atari/d_drive\n");
+#endif
+#ifdef ENABLE_LILO
+  printf("  -l, --lilo                 boot a linux kernel\n");
+#endif
 #ifndef FixedSizeFastRAM
   printf("  -F, --fastram SIZE         FastRAM size (in MB)\n");
 #endif
@@ -599,6 +637,7 @@ void preset_cfg() {
   preset_lilo();
   preset_midi();
   preset_nfcdroms();
+  preset_autozoom();
 }
 
 void postload_cfg() {
@@ -615,6 +654,7 @@ void postload_cfg() {
   postload_lilo();
   postload_midi();
   postload_nfcdroms();
+  postload_autozoom();
 }
 
 void presave_cfg() {
@@ -631,6 +671,7 @@ void presave_cfg() {
   presave_lilo();
   presave_midi();
   presave_nfcdroms();
+  presave_autozoom();
 }
 
 void early_cmdline_check(int argc, char **argv) {
@@ -670,7 +711,9 @@ int process_cmdline(int argc, char **argv)
 	while ((c = getopt_long (argc, argv,
 							 "a:" /* floppy image file */
 							 "e"  /* boot emutos */
+#ifdef ENABLE_LILO
 							 "l"  /* boot lilo */
+#endif
 #ifdef DEBUGGER
 							 "D"  /* debugger */
 #endif
@@ -682,7 +725,9 @@ int process_cmdline(int argc, char **argv)
 							 "v:" /* VIDEL refresh */
 							 "r:" /* resolution */
 							 "m:" /* attached monitor */
+#if HOSTFS_SUPPORT
 							 "d:" /* filesystem assignment */
+#endif
 							 "s"  /* save config file */
 
 							 "c:" /* path to config file */
@@ -717,9 +762,11 @@ int process_cmdline(int argc, char **argv)
 				bx_options.video.fullscreen = true;
 				break;
 
+#ifdef ENABLE_LILO
 			case 'l':
 				boot_lilo = true;
 				break;
+#endif
 
 			case 'v':
 				bx_options.video.refresh = atoi(optarg);
@@ -743,6 +790,7 @@ int process_cmdline(int argc, char **argv)
 				bx_options.video.boot_color_depth = atoi(optarg);
 				break;
 
+#if HOSTFS_SUPPORT
 			case 'd':
 				if ( strlen(optarg) < 4 || optarg[1] != ':') {
 					fprintf(stderr, "Not enough parameters for -d\n");
@@ -763,6 +811,7 @@ int process_cmdline(int argc, char **argv)
 					// Just make sure postload_cfg is called after this.
 				}
 				break;
+#endif
 
 			case 's':
 				saveConfigFile = true;
@@ -781,6 +830,50 @@ int process_cmdline(int argc, char **argv)
 	return optind;
 }
 
+void postload_PathTag( const char *filename, struct Config_Tag *ptr ) {
+	char *path = (char *)ptr->buf;
+	if ( !strlen(path) )
+		return;
+
+	char home[2048];
+	const char *prefix = NULL;
+	size_t prefixLen = 0;
+
+	if ( path[0] == '~' ) {
+		// replace with the home folder path
+		prefix = getHomeFolder(home, sizeof(home));
+		prefixLen = strlen( prefix );
+		path++;
+	} else if (path[0] == '*') {
+		prefix=getDataFolder(home, sizeof(home));
+		prefixLen = strlen( prefix );
+		path++;
+	} else if ( path[0] != '/' && path[0] != '\\' && path[1] != ':' ) {
+		// find the last dirseparator
+		const char *slash = &filename[strlen(filename)];
+		while ( --slash >= filename &&
+			*slash != '/' &&
+			*slash != '\\' );
+		// get the path part (in front of the slash)
+		prefix = filename;
+		prefixLen = slash - prefix + 1;
+	}
+
+	if ( prefixLen > 0 ) {
+		if (prefix[prefixLen-1]!=DIRSEPARATOR[0]) // if there isn't the last slash then add it 
+		{
+			prefixLen++;
+		}
+		if ( (size_t)ptr->buf_size >= prefixLen + strlen(path) ) {
+			memmove( (char*)ptr->buf + prefixLen, path, strlen(path)+1 );	
+			memmove( (char*)ptr->buf, prefix, prefixLen );
+			((char*)ptr->buf)[prefixLen-1]=DIRSEPARATOR[0];
+		} else {
+			fprintf(stderr, "Error - config entry size is insufficient\n" );
+		}
+	}
+}
+
 // append a filename to a path
 char *addFilename(char *buffer, const char *file, unsigned int bufsize)
 {
@@ -826,11 +919,15 @@ char *getDataFilename(const char *file, char *buffer, unsigned int bufsize)
 static int process_config(FILE *f, const char *filename, struct Config_Tag *conf, char *title, bool verbose)
 {
 	int status = input_config(filename, conf, title);
-	if (verbose) {
-		if (status >= 0)
+	if (status >= 0) {
+		if (verbose)
 			fprintf(f, "%s configuration: found %d valid directives.\n", title, status);
-		else
-			fprintf(f, "Error while reading/processing the '%s' config file.\n", filename);
+		struct Config_Tag *ptr;
+		for ( ptr = conf; ptr->buf; ++ptr )
+			if ( ptr->type == Path_Tag )
+				postload_PathTag(filename, ptr);
+	} else {
+		fprintf(f, "Error while reading/processing the '%s' config file.\n", filename);
 	}
 	return status;
 }
@@ -849,9 +946,7 @@ static bool decode_ini_file(FILE *f, const char *rcfile)
 
 	process_config(f, rcfile, global_conf, "[GLOBAL]", true);
 	process_config(f, rcfile, startup_conf, "[STARTUP]", true);
-#ifdef USE_JIT
 	process_config(f, rcfile, jit_conf, "[JIT]", true);
-#endif
 	process_config(f, rcfile, video_conf, "[VIDEO]", true);
 	process_config(f, rcfile, tos_conf, "[TOS]", true);
 	process_config(f, rcfile, ide_swap ? diskd_configs : diskc_configs, "[IDE0]", true);
@@ -878,6 +973,7 @@ static bool decode_ini_file(FILE *f, const char *rcfile)
 	process_config(f, rcfile, lilo_conf, "[LILO]", true);
 	process_config(f, rcfile, midi_conf, "[MIDI]", true);
 	process_config(f, rcfile, nfcdroms_conf, "[CDROMS]", true);
+	process_config(f, rcfile, autozoom_conf, "[AUTOZOOM]", true);
 
 	return true;
 }
@@ -898,9 +994,7 @@ bool saveSettings(const char *fs)
 		return false;
 	}
 	update_config(fs, startup_conf, "[STARTUP]");
-#ifdef USE_JIT
 	update_config(fs, jit_conf, "[JIT]");
-#endif
 	update_config(fs, video_conf, "[VIDEO]");
 	update_config(fs, tos_conf, "[TOS]");
 	update_config(fs, ide_swap ? diskd_configs : diskc_configs, "[IDE0]");
@@ -922,6 +1016,7 @@ bool saveSettings(const char *fs)
 	update_config(fs, lilo_conf, "[LILO]");
 	update_config(fs, midi_conf, "[MIDI]");
 	update_config(fs, nfcdroms_conf, "[CDROMS]");
+	update_config(fs, autozoom_conf, "[AUTOZOOM]");
 
 	return true;
 }
