@@ -59,9 +59,9 @@ bool cpu_debugging = false;
 struct flag_struct regflags;
 
 /* LongJump buffers */
-jmp_buf excep_env;
+JMP_BUF excep_env;
 #ifdef DISDIP
-jmp_buf loop_env;
+JMP_BUF loop_env;
 #endif
 /* Opcode of faulting instruction */
 uae_u16 last_op_for_exception_3;
@@ -93,6 +93,8 @@ struct rec_step {
 	uae_u32 a[8];
 	uae_u32 pc;
 	uae_u16 sr;
+	uae_u32 msp;
+	uae_u32 isp;
 };
 
 const int LOG_SIZE = 8192;
@@ -114,6 +116,8 @@ void m68k_record_step(uaecptr pc)
 	log[log_ptr].pc = pc;
 	MakeSR();
 	log[log_ptr].sr = regs.sr;
+	log[log_ptr].msp = regs.msp;
+	log[log_ptr].isp = regs.isp;
 	log_ptr = (log_ptr + 1) % LOG_SIZE;
 }
 
@@ -124,7 +128,7 @@ static void dump_log(void)
 		return;
 	for (int i = 0; i < LOG_SIZE; i++) {
 		int j = (i + log_ptr) % LOG_SIZE;
-		fprintf(f, "pc %08x %04x\n", log[j].pc, log[j].sr);
+		fprintf(f, "pc %08x sr %04x msp %08x isp %08x\n", log[j].pc, log[j].sr, log[j].msp, log[j].isp);
 		fprintf(f, "d0 %08x d1 %08x d2 %08x d3 %08x\n", log[j].d[0], log[j].d[1], log[j].d[2], log[j].d[3]);
 		fprintf(f, "d4 %08x d5 %08x d6 %08x d7 %08x\n", log[j].d[4], log[j].d[5], log[j].d[6], log[j].d[7]);
 		fprintf(f, "a0 %08x a1 %08x a2 %08x a3 %08x\n", log[j].a[0], log[j].a[1], log[j].a[2], log[j].a[3]);
@@ -747,7 +751,11 @@ void Exception(int nr, uaecptr oldpc)
 	exc_push_word(0);	/* WB2S */
 	exc_push_word(0);	/* WB3S */
 	exc_push_word(regs.mmu_ssw);
+#ifdef PUREC
 	exc_push_long(0 /* was regs.mmu_fault_addr */);	/* EA *//* bullshit here, took 10 hours to debug with PureC $12345678 test. It should be an internal register, keep 0 to preserve MSP in PureC */
+#else
+	exc_push_long(regs.mmu_fault_addr); /* EA *//* ARAnyM has 040, so stack format 7, not 000 stack format */
+#endif
 	exc_make_frame(7, regs.sr, currpc, 2, 0, 0);
 
 	building_bus_fault_stack_frame=0;
@@ -1264,7 +1272,7 @@ void REGPARAM2 op_illg (uae_u32 opcode)
 	if ((opcode & 0xF000) == 0xA000) {
 	Exception(0xA,0);
 #ifdef DISDIP
-	longjmp(loop_env, 0);
+	LONGJMP(loop_env, 0);
 #else
 	return;
 #endif
@@ -1273,7 +1281,7 @@ void REGPARAM2 op_illg (uae_u32 opcode)
 	if ((opcode & 0xF000) == 0xF000) {
 	Exception(0xB,0);
 #ifdef DISDIP
-	longjmp(loop_env, 0);
+	LONGJMP(loop_env, 0);
 #else
 	return;
 #endif
@@ -1287,7 +1295,7 @@ void REGPARAM2 op_illg (uae_u32 opcode)
 	Exception (4,0);
 #ifdef DISDIP
 }
-	longjmp(loop_env, 0);
+	LONGJMP(loop_env, 0);
 #else
 	return;
 #endif
@@ -1425,7 +1433,7 @@ int m68k_do_specialties(void)
 
 		SERVE_INTERNAL_IRQ();
 		SERVE_VBL_MFP(true);
-		if (SPCFLAGS_TEST( SPCFLAG_NMI ))
+		if (SPCFLAGS_TEST( SPCFLAG_BRK ))
 			break;
 	}
 	if (SPCFLAGS_TEST( SPCFLAG_TRACE ))
@@ -1440,14 +1448,11 @@ int m68k_do_specialties(void)
 		regs.spcflags |= SPCFLAG_DOINT;
 	}
 */
-	if (SPCFLAGS_TEST( SPCFLAG_BRK | SPCFLAG_MODE_CHANGE )) {
-		SPCFLAGS_CLEAR( SPCFLAG_BRK | SPCFLAG_MODE_CHANGE );
+	if (SPCFLAGS_TEST( SPCFLAG_BRK /*| SPCFLAG_MODE_CHANGE*/ )) {
+		SPCFLAGS_CLEAR( SPCFLAG_BRK /*| SPCFLAG_MODE_CHANGE*/ );
 		return 1;
 	}
 
-	if (SPCFLAGS_TEST( SPCFLAG_NMI ))
-		return 1;
-
 	return 0;
 }
 
@@ -1456,7 +1461,7 @@ void m68k_do_execute (void)
     uae_u32 pc;
     uae_u32 opcode;
 #ifdef DISDIP
-    if (setjmp(loop_env)) return;
+    if (SETJMP(loop_env)) return;
 #endif	    
     for (;;) {
 	pc = m68k_getpc();
@@ -1501,7 +1506,7 @@ void m68k_do_execute (void)
 void m68k_compile_execute (void)
 {
 setjmpagain:
-    int prb = setjmp(excep_env);
+    int prb = SETJMP(excep_env);
     if (prb != 0) {
 	flush_icache(0);
         Exception(prb, 0);
@@ -1533,7 +1538,7 @@ void m68k_execute (void)
 #endif
 
 setjmpagain:
-    int prb = setjmp(excep_env);
+    int prb = SETJMP(excep_env);
     if (prb != 0) {
         Exception(prb, 0);
 #ifdef DEBUGGER
@@ -1614,7 +1619,7 @@ void m68k_disasm (uaecptr addr, uaecptr *nextpc, int cnt)
 	    opcode = 0x4AFC;
 	}
 	dp = table68k + opcode;
-	for (lookup = lookuptab;lookup->mnemo != dp->mnemo; lookup++)
+	for (lookup = lookuptab; (unsigned int)lookup->mnemo != dp->mnemo; lookup++)
 	    ;
 
 	strcpy (instrname, lookup->name);
@@ -1657,13 +1662,13 @@ void m68k_disasm (uaecptr addr, uaecptr *nextpc, int cnt)
 void newm68k_disasm(FILE *f, uaecptr addr, uaecptr *nextpc, volatile unsigned int cnt)
 {
     char *buffer = (char *)malloc(80 * sizeof(char));
-    jmp_buf excep_env_old;
-    memcpy(excep_env_old, excep_env, sizeof(jmp_buf));
+    JMP_BUF excep_env_old;
+    memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
     strcpy(buffer,"");
     volatile uaecptr newpc = 0;
     m68kpc_offset = addr - m68k_getpc ();
     if (cnt == 0) {
-        int prb = setjmp(excep_env);
+        int prb = SETJMP(excep_env);
         if (prb != 0) {
             goto setjmpagainx;
         }
@@ -1681,7 +1686,7 @@ void newm68k_disasm(FILE *f, uaecptr addr, uaecptr *nextpc, volatile unsigned in
             opcode = 0x4AFC;
         }
         dp = table68k + opcode;
-        for (lookup = lookuptab;lookup->mnemo != dp->mnemo; lookup++)
+        for (lookup = lookuptab;(unsigned int)lookup->mnemo != dp->mnemo; lookup++)
             ;
         strcpy (instrname, lookup->name);
         ccpt = strstr (instrname, "cc");
@@ -1700,7 +1705,7 @@ void newm68k_disasm(FILE *f, uaecptr addr, uaecptr *nextpc, volatile unsigned in
         }
     } else {
 setjmpagain:
-        int prb = setjmp(excep_env);
+        int prb = SETJMP(excep_env);
         if (prb != 0) {
 		fprintf (f, " unknown address\n");
                 goto setjmpagain;
@@ -1721,7 +1726,7 @@ setjmpagain:
 			opcode = 0x4AFC;
 		}
 		dp = table68k + opcode;
-		for (lookup = lookuptab;lookup->mnemo != dp->mnemo; lookup++)
+		for (lookup = lookuptab;(unsigned int)lookup->mnemo != dp->mnemo; lookup++)
 		    ;
 		strcpy (instrname, lookup->name);
 		ccpt = strstr (instrname, "cc");
@@ -1764,7 +1769,7 @@ setjmpagainx:
     if (nextpc)
 	*nextpc = m68k_getpc () + m68kpc_offset;
     free(buffer);
-    memcpy(excep_env, excep_env_old, sizeof(jmp_buf));
+    memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 
 #ifdef FULL_HISTORY
@@ -1781,11 +1786,11 @@ void showDisasm(uaecptr addr) {
 		buff[i] = (char *)malloc(80 * sizeof(char));
 		strcpy(buff[i],"");
 	}
-	jmp_buf excep_env_old;
-	excep_env_old = excep_env;
-	uaecptr newpc = 0;
+	JMP_BUF excep_env_old;
+	memcpy(excep_env_old, excep_env, sizeof(JMP_BUF));
+	volatile uaecptr newpc = 0;
 	m68kpc_offset = addr - m68k_getpc ();
-	int prb = setjmp(excep_env);
+	int prb = SETJMP(excep_env);
 	if (prb != 0) {
 		bug("%s%s%s%s%s%s%s%s%s%s%s%s unknown address", sbuffer[0], buff[0],  buff[1],  buff[2], buff[3], buff[4], sbuffer[1], sbuffer[2], sbuffer[3], sbuffer[4], sbuffer[5], sbuffer[6]);
 		free(buffer);
@@ -1808,7 +1813,7 @@ void showDisasm(uaecptr addr) {
 		opcode = 0x4AFC;
 	}
 	dp = table68k + opcode;
-	for (lookup = lookuptab;lookup->mnemo != dp->mnemo; lookup++)
+	for (lookup = lookuptab;(unsigned int)lookup->mnemo != dp->mnemo; lookup++)
 		    ;
 	strcpy (instrname, lookup->name);
 	ccpt = strstr (instrname, "cc");
@@ -1846,7 +1851,7 @@ void showDisasm(uaecptr addr) {
 	free(buffer);
 	for (int i = 0; i < 7; i++) free(sbuffer[i]);
 	for (int i = 0; i < 5; i++) free(buff[i]);
-	excep_env = excep_env_old;
+	memcpy(excep_env, excep_env_old, sizeof(JMP_BUF));
 }
 #endif
 #endif
